<?php

namespace App\Finance\Services;

use App\Finance\Repositories\Contracts\InvoiceRecordRepository;
use App\Finance\Repositories\Eloquent\BhuaInvoiceItemsRepositoryEloquent;
use App\Finance\Repositories\Eloquent\BhuaInvoiceRepositoryEloquent;
use App\Finance\Repositories\Eloquent\RhawnInvoiceItemsRepositoryEloquent;
use App\Finance\Repositories\Eloquent\RhawnInvoiceRepositoryEloquent;
use App\Finance\Services\Api\BaiwangApiService;
use App\Mailer\MailService;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Repositories\Transformers\Finance\InvoiceRecordStatusDetailTransformer;
use App\Repositories\Transformers\Finance\InvoiceRecordTransformer;
use App\Services\ThirdPlatform\PlatformBaseService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\Helpers;
use Illuminate\Support\Facades\Storage;

class InvoiceService
{
    use Helpers;

    public function __construct(InvoiceRecordRepository $invoiceRecordRepository)
    {
        $this->apiService = (new BaiwangApiService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnInvoiceRepository = app(RhawnInvoiceRepositoryEloquent::class);
        $this->rhawnInvoiceItemsRepository = app(RhawnInvoiceItemsRepositoryEloquent::class);
        $this->bhuaInvoiceRepository = app(BhuaInvoiceRepositoryEloquent::class);
        $this->bhuaInvoiceItemsRepository = app(BhuaInvoiceItemsRepositoryEloquent::class);
        $this->invoiceRecordRepository = $invoiceRecordRepository;
        $this->mailService = app(MailService::class);
    }

    /**
     * 注册平台信息
     * @param string $platformName
     */
    public function registerPlatform($platformName = 'baiwang')
    {
        if('baiwang' == $platformName){
            $registerInfo = [
                'platform_name' => 'baiwang_invoice',
                'platform_title' => '百旺开票',
                'platform_url' => 'http://api.baiwangjs.com/bwdpzx/agentinvoiceservice-cloud/agentiscloud',
                'platform_desc' => '百旺开票对接',
                'platform_params' => [
                    'test_url' => 'http://dlkp.kjgc.htjs.net/agentiscloud',
                    'rhawn' => [
                        'qyid' => '23020214385104316',
                        'nsrsbh' => '91310120MA1HMP790Y',
                        'nsrmc' => '上海易恩化学技术有限公司',
                        'nsrdz' => '上海市奉贤区银工路28号E幢',
                        'nsrdh' => '021-57461626',
                        'nsryh' => '交行上海南桥支行',
                        'nsrzh' => '310069309018800003232',
                        'key' => '4hCGzt6uETEBIDVpdxtT25lM',
                        'secret' => '6x5KNgZmu9fHI2o1Ni5ecfI7y2L6JJ',
                        'tax_rate' => '13'
                    ],
                    'bhua' => [
                        'qyid' => '23022310311905803',
                        'nsrsbh' => '91310115312498004F',
                        'nsrmc' => '上海百舜生物科技有限公司',
                        'nsrdz' => '上海市奉贤区联合北路215号第3幢1242室 ',
                        'nsrdh' => '021-37581181',
                        'nsryh' => '交行上海大场支行',
                        'nsrzh' => '310065060018010017377',
                        'key' => '4hCGzt6uETEBIDVpdxtT25lM',
                        'secret' => '6x5KNgZmu9fHI2o1Ni5ecfI7y2L6JJ',
                        'tax_rate' => '13'
                    ]
                ],
                'platform_type' => 'third',
            ];
        }

        app(PlatformBaseService::class)->registerPlatform($registerInfo);
    }

    /**
     * 申请开票（包括蓝字和红字）
     * @param $invoiceParams
     */
    public function applyInvoice($invoiceParams)
    {
        if(!isset($invoiceParams['invoice_platform']) || empty($invoiceParams['invoice_platform'])){
            throw new \Exception('开票平台不存在',502);
        }
        if(!in_array($invoiceParams['invoice_platform'],['rhawn','bhua'])){
            throw new \Exception('开票平台不在允许范围内',502);
        }

        if(!isset($invoiceParams['invoice_type']) || empty($invoiceParams['invoice_type'])){
            throw new \Exception('开票类型不存在',502);
        }

        if(!in_array($invoiceParams['invoice_type'],['blue','red'])){
            throw new \Exception('开票类型错误',502);
        }

        if(!isset($invoiceParams['invoice_category'])){
            throw new \Exception('开票种类不存在',502);
        }

        if(!in_array($invoiceParams['invoice_category'],['1','2'])){
            throw new \Exception('开票种类错误',502);
        }

        if(!isset($invoiceParams['invoice_customer']) || empty($invoiceParams['invoice_customer'])){
            throw new \Exception('购买方发票信息不存在',502);
        }

        //设置发票平台
        $this->apiService->setInvoicePlatform($invoiceParams['invoice_platform']);

        //查询是否存在发票记录
        $invoiceInfo = null;
        switch($invoiceParams['invoice_platform']){
            case 'rhawn' : $invoiceInfo = $this->rhawnInvoiceRepository->getInvoiceById($invoiceParams['invoice_platform_id']);
                break;
            case 'bhua' : $invoiceInfo = $this->bhuaInvoiceRepository->getInvoiceById($invoiceParams['invoice_platform_id']);
                break;
        }
        if(!$invoiceInfo){
            $errorMessage = $invoiceParams['invoice_platform'].' invoice_platform_id ：'.$invoiceParams['invoice_platform_id'].' 没有找到相应的开票信息';
            SimpleLogs::writeLog($errorMessage, __CLASS__.':'.__FUNCTION__, 'error');
            throw new \Exception('没有找到相应的开票信息',502);
        }

        //对发票项进行检查
        /*if(!empty($invoiceParams['invoice_items'])){
            $itemIds = [];
            $invoiceItemsParams = [];
            foreach($invoiceParams['invoice_items'] as $item){
                $invoiceItemsParams[$item['item_id']] = $item;
                array_push($itemIds,$item['item_id']);
            }
            //查询产品是否存在
            $invoiceItem = $this->rhawnInvoiceItemsRepository->getInvoiceItemsByIdAndItemId($invoiceParams['invoice_platform_id'],$itemIds);
            if($invoiceItem){
                foreach($invoiceItem as $item){
                    if(isset($invoiceItemsParams[$item['soid_id']])){
                        if($invoiceItemsParams[$item['soid_id']]['item_nums'] > $item['dpd_num']){
                            SimpleLogs::writeLog('item_id : '.$item['soid_id'].' 开票数量超出已发货数量', __CLASS__.':'.__FUNCTION__, 'error');
                            throw new \Exception('开票数量超出已发货数量',502);
                        }
                    }
                }
            }
        }*/

        $invoiceNumber = 'I';
        $invoiceNumber .= $invoiceParams['invoice_platform'] == 'rhawn' ? 'R' : 'B';
        $invoiceNumber .= $invoiceParams['invoice_type'] == 'blue' ? 'B' : 'R';
        $invoiceNumber .= $invoiceParams['invoice_category'] . date('Ymd',time()).strtoupper($this->randomFromDevice(5));

        $platformInfo = $this->apiService->getPlatformInfo()['platform_params'][$invoiceParams['invoice_platform']];

        //组装请求参数
        $applyInvoiceParams = [];
        $applyInvoiceParams['qyId'] = $platformInfo['qyid'];
        $applyInvoiceParams['sksbh'] = $platformInfo['nsrsbh'];
        $applyInvoiceParams['bmbBbh'] = '';
        $applyInvoiceParams['ddlsh'] = $invoiceNumber;
        $applyInvoiceParams['fplxdm'] = $invoiceParams['invoice_category'] == '2' ? '01' : '02';
        $applyInvoiceParams['kplx'] = $invoiceParams['invoice_type'] == 'blue' ? 0 : 1;
        $applyInvoiceParams['tspz'] = '';
        $applyInvoiceParams['zsfs'] = 0;
        $applyInvoiceParams['qdbz'] = count($invoiceParams['invoice_items']) > 8 ? 1 : 0;
        $applyInvoiceParams['xsfMc'] = $platformInfo['nsrmc']; // 销售方名称
        $applyInvoiceParams['xsfNsrsbh'] = $platformInfo['nsrsbh']; //销售方纳税人识别号
        $applyInvoiceParams['xsfDz'] = $platformInfo['nsrdz']; // 销售方地址
        $applyInvoiceParams['xsfDh'] = $platformInfo['nsrdh']; // 销售方电话
        $applyInvoiceParams['xsfYh'] = $platformInfo['nsryh']; // 销售方银行名称
        $applyInvoiceParams['xsfZh'] = $platformInfo['nsrzh']; // 销售方银行账户
        $applyInvoiceParams['gmfMc'] = $invoiceParams['invoice_customer']['cust_name']; // 购买方名称
        $applyInvoiceParams['gmfNsrsbh'] = $invoiceParams['invoice_customer']['cust_code']; // 购买方纳税人识别号
        $applyInvoiceParams['gmfDz'] = $invoiceParams['invoice_customer']['cust_address']; // 购买方地址
        $applyInvoiceParams['gmfDh'] = $invoiceParams['invoice_customer']['cust_phone']; // 购买方电话
        $applyInvoiceParams['gmfYh'] = $invoiceParams['invoice_customer']['cust_bankname']; // 购买方银行名称
        $applyInvoiceParams['gmfZh'] = $invoiceParams['invoice_customer']['cust_bankaccount']; // 购买方银行账户
        $applyInvoiceParams['gmfMobile'] = '';//$invoiceParams['invoice_customer']['cust_mobile']; // 购买方银行账户
        $applyInvoiceParams['gmfEmail'] = '';//$invoiceParams['invoice_customer']['cust_email']; // 购买方邮箱

        $applyInvoiceParams['sqr'] = $invoiceParams['invoice_creater_by'];
        $applyInvoiceParams['kpr'] = $invoiceParams['invoice_creater_by'];
        $applyInvoiceParams['dlzh'] = $platformInfo['nsrsbh'];
        $applyInvoiceParams['sfwzzfp'] = 'N';
        $applyInvoiceParams['zpFppzdm'] = $invoiceParams['invoice_category'] == 2 ? '1130' : '04';
        $applyInvoiceParams['jshj'] = bcmul($invoiceParams['invoice_money'],1,2);
        $applyInvoiceParams['hjje'] = bcdiv($applyInvoiceParams['jshj'],1.13,2);
        $applyInvoiceParams['hjse'] = bcsub($applyInvoiceParams['jshj'],$applyInvoiceParams['hjje'],2);//bcmul($applyInvoiceParams['hjje'],bcdiv($platformInfo['tax_rate'],100,3),2);

        //初始化非必填项
        $applyInvoiceParams['bz'] = $invoiceParams['invoice_remark'];
        $applyInvoiceParams['kce'] = '';
        $applyInvoiceParams['sslkjly'] = '';
        //是否是事业单位
        $applyInvoiceParams['spflxconfirm'] = $invoiceParams['invoice_customer']['cust_spflxconfirm'];

        $applyInvoiceParams['detailParam'] = [];
        //开票明细列
        foreach($invoiceParams['invoice_items'] as $item){
            $applyInvoiceItems = [];
            $applyInvoiceItems['fphxz'] = 0;
            $applyInvoiceItems['hsbz'] = 1; //默认含税
            $applyInvoiceItems['spmc'] = $item['item_name'];//$this->charsetToGBK($item['item_name']);
            $applyInvoiceItems['spbm'] = '1070214010000000000';//$item['p_code'];
            $applyInvoiceItems['ggxh'] = $item['item_spec'];
            $applyInvoiceItems['dw'] = $item['item_unit'];
            $applyInvoiceItems['je'] = bcmul($item['item_price'],1,2);
            $applyInvoiceItems['sl'] = $item['item_nums'];
            $applyInvoiceItems['dj'] = bcdiv($applyInvoiceItems['je'],$applyInvoiceItems['sl'],2);
            $applyInvoiceItems['se'] = '';
            $applyInvoiceItems['slv'] = bcdiv($platformInfo['tax_rate'],100,3);
            $applyInvoiceItems['lslbs'] = 0;
            $applyInvoiceItems['yhzcbs'] = 0;
            $applyInvoiceItems['zxbm'] = '';
            $applyInvoiceItems['zzstsgl'] = '';
            array_push($applyInvoiceParams['detailParam'],$applyInvoiceItems);
        }

        //请求接口之前记录开票信息
        $invoiceParams = array_merge($invoiceParams,['invoice_number' => $invoiceNumber]);
        $result = $this->saveInvoiceRecord($invoiceParams);
        if($result){
            //记录请求参数
            $this->apiService->storePlatformDataEntries('invoice_apply_'.$result['invoice_number'],$applyInvoiceParams);
            try{
                $response = $this->apiService->applyElecInvoice($applyInvoiceParams);
                if($response){
                    $this->invoiceRecordRepository->saveInvoice([
                        'invoice_number' => $result['invoice_number'],
                        'invoice_status' => 0
                    ]);
                }
            }catch(\Throwable $exception){
                $update = [];
                $update['invoice_number'] = $result['invoice_number'];
                $update['api_response'] = $exception->getMessage();
                $this->saveInvoiceRecord($update);
            }
        }

        throw new \Exception('申请发票失败',502);
    }

    public function cancelBlueInvoice()
    {
        if(!isset($invoiceParams['invoice_platform']) || empty($invoiceParams['invoice_platform'])){
            throw new \Exception('开票平台不存在',502);
        }
        if(!isset($invoiceParams['invoice_real_number']) || empty($invoiceParams['invoice_real_number'])){
            throw new \Exception('发票号码不存在',502);
        }

        if(!isset($invoiceParams['invoice_cancel_by']) || empty($invoiceParams['invoice_cancel_by'])){
            throw new \Exception('发票作废人不存在',502);
        }

        if(!isset($invoiceParams['invoice_platform']) || empty($invoiceParams['invoice_platform'])){
            throw new \Exception('开票平台不存在',502);
        }
        if(!in_array($invoiceParams['invoice_platform'],['rhawn','bhua'])){
            throw new \Exception('开票平台不在允许范围内',502);
        }

        //设置发票平台
        $this->apiService->setInvoicePlatform($invoiceParams['invoice_platform']);

        //查询发票号码对应的数据
        $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByRealNumber($invoiceParams['invoice_real_number']);
        if(!$invoiceInfo){
            throw new \Exception('发票数据不存在',502);
        }

        if($invoiceInfo['invoice_status'] != 1){
            throw new \Exception('该发票不是确认状态,无法作废',502);
        }

        $platformInfo = $this->apiService->getPlatformInfo()['platform_params'][$invoiceParams['invoice_platform']];

        $cancelInvoiceParams = [];
        $cancelInvoiceParams['qyId'] = $platformInfo['qyid'];
        $cancelInvoiceParams['ddlsh'] = $invoiceInfo['invoice_number'];
        $cancelInvoiceParams['dlzh'] = $platformInfo['nsrsbh'];
        $cancelInvoiceParams['zfr'] = $invoiceParams['invoice_real_number'];

        //记录请求参数
        $this->apiService->storePlatformDataEntries('invoice_cancel_'.$invoiceInfo['invoice_number'].'_'.$invoiceInfo['invoice_real_number'],$cancelInvoiceParams);
        $response = $this->apiService->cancelElecInvoice($cancelInvoiceParams);
        if($response){
            $this->invoiceRecordRepository->saveInvoice([
                'invoice_number' => $invoiceInfo['invoice_number'],
                'invoice_status' => 2
            ]);
            return true;
        }
        throw new \Exception('发票作废失败',502);
    }

    public function confirmRedInvoice()
    {
    }

    public function cancelRedInvoice()
    {
    }

    public function queryInvoiceInfo($invoiceParams)
    {
        //设置发票平台
        $this->apiService->setInvoicePlatform($invoiceParams['invoice_platform']);

        $cancelInvoiceParams = [];
        $cancelInvoiceParams['ddlsh'] = $invoiceParams['invoice_number'];

        //记录请求参数
        $response = $this->apiService->queryOrder($cancelInvoiceParams);
        return $response;
    }

    public function getQiyeInvoiceInfo()
    {
        //设置发票平台
        $info = $this->apiService->queryQiyeUniqueSign('rhawn');
        var_dump($info);
    }

    /**
     * 保存开票记录
     * @param $params
     */
    public function saveInvoiceRecord($invoiceParams)
    {
        $saveParams = [];
        try{
            $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByNumber($invoiceParams['invoice_number']);
            if(!$invoiceInfo){
                $saveParams['invoice_number'] = $invoiceParams['invoice_number'];
                $saveParams['invoice_type'] = $invoiceParams['invoice_type'] == 'blue' ? 1 : 2;
                $saveParams['invoice_status'] = -1;
                $saveParams['invoice_money'] = $invoiceParams['invoice_money'];
                $saveParams['invoice_platform'] = $invoiceParams['invoice_platform'];
                $saveParams['platform_invoice_id'] = $invoiceParams['invoice_platform_id'];
                $saveParams['invoice_creater_by'] = $invoiceParams['invoice_creater_by'];
                $saveParams['invoice_category'] = $invoiceParams['invoice_category'];
                $saveParams['invoice_items'] = json_encode($invoiceParams['invoice_items'],JSON_UNESCAPED_UNICODE);
                $saveParams['invoice_customer'] = json_encode($invoiceParams['invoice_customer'],JSON_UNESCAPED_UNICODE);

                $saveParams['api_response'] = '';
            }else{
                $saveParams = current($invoiceInfo);

                if(isset($invoiceParams['invoice_type']) && !empty($invoiceParams['invoice_type'])){
                    $saveParams['invoice_type'] = $invoiceParams['invoice_type'] == 'blue' ? 1 : 2;
                }
                if(isset($invoiceParams['invoice_status']) && !empty($invoiceParams['invoice_status'])){
                    $saveParams['invoice_status'] = $invoiceParams['invoice_status'];
                }
                if(isset($invoiceParams['invoice_money']) && !empty($invoiceParams['invoice_money'])){
                    $saveParams['invoice_money'] = $invoiceParams['invoice_money'];
                }
                if(isset($invoiceParams['invoice_platform']) && !empty($invoiceParams['invoice_platform'])){
                    $saveParams['invoice_platform'] = $invoiceParams['invoice_platform'];
                }
                if(isset($invoiceParams['platform_invoice_id']) && !empty($invoiceParams['platform_invoice_id'])){
                    $saveParams['platform_invoice_id'] = $invoiceParams['platform_invoice_id'];
                }
                if(isset($invoiceParams['invoice_creater_by']) && !empty($invoiceParams['invoice_creater_by'])){
                    $saveParams['invoice_creater_by'] = $invoiceParams['invoice_creater_by'];
                }
                if(isset($invoiceParams['invoice_category']) && !empty($invoiceParams['invoice_category'])){
                    $saveParams['invoice_category'] = $invoiceParams['invoice_category'];
                }
                if(isset($invoiceParams['invoice_items']) && !empty($invoiceParams['invoice_items'])){
                    $saveParams['invoice_items'] = $invoiceParams['invoice_items'];
                }
                if(isset($invoiceParams['invoice_customer']) && !empty($invoiceParams['invoice_customer'])){
                    $saveParams['invoice_customer'] = $invoiceParams['invoice_customer'];
                }

                if(isset($invoiceParams['api_response']) && !empty($invoiceParams['api_response'])){
                    $saveParams['api_response'] = $invoiceParams['api_response'];
                }
            }

            $result = $this->invoiceRecordRepository->saveInvoice($saveParams);

            if($result){
                return $result;
            }
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':'.__FUNCTION__, 'error');
            return false;
        }
    }

    public function apiCallBack($params)
    {
        try{
            if(isset($params) && !empty($params)){
                $queueParams = [];
                //查询流水号是否有效
                $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByNumber($params['ddlsh']);
                if(!$invoiceInfo){
                    //SimpleLogs::writeLog($params, __CLASS__.':'.__FUNCTION__.' 无法找到发票流水号：'.$params['ddlsh'].'的数据', 'error');
                    return false;
                }

                //现将返回的数据进行存储
                switch($params['ywlxdm']){
                    case 'fpkj' :
                        $this->apiService->storePlatformDataEntries('invoice_callBack_'.$params['ddlsh'].'_applyInvoice',$params);
                        $queueParams = ['params' => $params,'type' => 'applyInvoiceCallBack'];
                        break;
                }
                //塞入队列
                $this->apiService->pushQueue([
                    'params' => $queueParams,
                    'consumer' => __CLASS__,
                    'method' => 'invoiceApiCallBack'
                ],'invoiceCallBack');
            }
            /*$values = $this->apiService->getPlatformDataEntries('invoice_callBack_IRB220230301KOOA9_applyInvoice','data_values');
            $queueParams = ['params' => $values,'type' => 'applyInvoiceCallBack'];
            $this->apiService->pushQueue([
                'params' => $queueParams,
                'consumer' => __CLASS__,
                'method' => 'invoiceApiCallBack'
            ],'invoiceCallBack');*/
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($params, __CLASS__.':'.__FUNCTION__.' params => ', 'error');
            SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':'.__FUNCTION__, 'error');
        }
    }

    public function tryAgainInvoiceApiResult()
    {
        $result = $this->invoiceRecordRepository->getNotResultInvoiceRecord();
        if($result){
            foreach($result as $res){
                $invoiceResponse = $this->queryInvoiceInfo([
                    'invoice_platform' => $res['invoice_platform'],
                    'invoice_number' => $res['invoice_number']
                ]);
                if(!$invoiceResponse){
                    //如果查询不到将状态更新
                    $this->invoiceRecordRepository->saveInvoice([
                        'invoice_number' => $res['invoice_number'],
                        'invoice_status' => 5
                    ]);
                    SimpleLogs::writeLog('当前票据 ['.$res['invoice_number'].'] 无法在开票系统中查询亦或已删除 ', __CLASS__.':'.__FUNCTION__);
                    continue;
                }
                if(in_array($invoiceResponse['status'],[0,1])){
                    $status = null;
                    switch($invoiceResponse['status']){
                        case 0 : $status = '待审核';break;
                        case 1 : $status = '开票中';break;
                    }
                    SimpleLogs::writeLog('当前票据 ['.$res['invoice_number'].'] 状态为 : '.$status, __CLASS__.':'.__FUNCTION__);
                   continue;
               }
               $params = [];
               $params['ddlsh'] = $res['invoice_number'];
               $params['status'] = $invoiceResponse['status'];
               switch($invoiceResponse['status']){
                   case 2 : $params['ycyy'] = '';$params['status'] = 1;break;
                   case 3 : $params['ycyy'] = $invoiceResponse['ycyy'];$params['status'] = 2;break;
               }
               $this->apiService->storePlatformDataEntries('invoice_callBack_'.$res['invoice_number'].'_applyInvoice',$params);
               $queueParams = ['params' => $params,'type' => 'applyInvoiceCallBack'];
               //塞入队列
               $this->apiService->pushQueue([
                   'params' => $queueParams,
                   'consumer' => __CLASS__,
                   'method' => 'invoiceApiCallBack'
               ],'invoiceCallBack');
           }
        }
    }

    public function invoiceApiCallBack($params)
    {
        if(empty($params)){
            return false;
        }
        //记录日志
        SimpleLogs::writeLog($params, __CLASS__.':'.__FUNCTION__.' callback params => ');

        //根据type回调相应方法处理逻辑
        if($params['type'] && method_exists(__CLASS__,$params['type'])){
            $func = $params['type'];
            $this->$func($params['params']);
        }
    }

    public function applyInvoiceCallBack($callBackParams)
    {
        if(empty($callBackParams)){
            return false;
        }

        //根据发票流水找到相应数据
        if(isset($callBackParams['ddlsh']) && !empty($callBackParams['ddlsh'])){
            $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByNumber($callBackParams['ddlsh']);
            if(!$invoiceInfo){
                SimpleLogs::writeLog($callBackParams, __CLASS__.':'.__FUNCTION__.' 无法找到发票流水号：'.$callBackParams['ddlsh'].'的数据', 'error');
                return false;
            }
            $invoiceInfo = current($invoiceInfo);
            //查询发票信息
            $invoiceResponse = $this->queryInvoiceInfo([
                'invoice_platform' => $invoiceInfo['invoice_platform'],
                'invoice_number' => $callBackParams['ddlsh']
            ]);

            //更新对应的发票数据，并反写回相应的平台发票
            try{
                $updateInvoice = [];
                $updateInvoice['invoice_number'] = $invoiceResponse['ddlsh'];
                if($callBackParams['status'] == 1 && $invoiceResponse['status'] == 2){
                    $updateInvoice['invoice_status'] = 1;
                    $updateInvoice['invoice_real_number'] = $invoiceResponse['qdfphm'];
                    $updateInvoice['invoice_real_date'] = strtotime($invoiceResponse['kprq']);
                    $updateInvoice['invoice_pdf_url'] = $invoiceResponse['pdf'];

                    $updatedInvoiceResult = $this->invoiceRecordRepository->saveInvoice($updateInvoice);

                    if($updatedInvoiceResult){
                        //对平台的发票信息进行更新
                        $updateResult = null;
                        $updateParams = [
                            'soi_api_status' => 1,
                            'soi_no' => $updatedInvoiceResult['invoice_real_number'],
                            'soi_date' => $updatedInvoiceResult['invoice_real_date'],
                            //'soi_total' => bcmul($callBackParams['jshj'],1,2),
                            'soi_tax' => bcmul($invoiceResponse['hjse'],1,2),
                            'soi_amount' => bcmul($invoiceResponse['hjje'],1,2)
                        ];
                        switch($updatedInvoiceResult['invoice_platform']){
                            case 'rhawn' :
                                $updateResult = $this->rhawnInvoiceRepository->updateInvoiceById($updatedInvoiceResult['platform_invoice_id'], $updateParams);break;
                            case 'bhua' :
                                $updateResult = $this->bhuaInvoiceRepository->updateInvoiceById($updatedInvoiceResult['platform_invoice_id'], $updateParams);break;
                        }
                        if($updateResult){
                            //塞入队列
                            $this->apiService->pushQueue([
                                'params' => $updatedInvoiceResult,
                                'consumer' => __CLASS__,
                                'method' => 'sendInvoiceEmail'
                            ],'invoiceEmailPush');
                            //$this->sendInvoiceEmail($updatedInvoiceResult);
                            $this->apiService->removePlatformDataEntries('invoice_callBack_'.$invoiceResponse['ddlsh'].'_applyInvoice');
                        }
                    }
                }
                else{
                    $updateInvoice['invoice_status'] = 2;
                    $updateInvoice['api_response'] = $callBackParams['ycyy'];
                    $updatedInvoiceResult = $this->invoiceRecordRepository->saveInvoice($updateInvoice);
                    if($updatedInvoiceResult) {
                        switch($updatedInvoiceResult['invoice_platform']){
                            case 'rhawn' :
                                $updateResult = $this->rhawnInvoiceRepository->updateInvoiceById($updatedInvoiceResult['platform_invoice_id'], [
                                    'soi_api_status' => 2,
                                    'soi_no' => mb_substr($callBackParams['ycyy'],0,250,'utf-8')
                                ]);break;
                            case 'bhua' :
                                $updateResult = $this->bhuaInvoiceRepository->updateInvoiceById($updatedInvoiceResult['platform_invoice_id'], [
                                    'soi_api_status' => 2,
                                    'soi_no' => mb_substr($callBackParams['ycyy'],0,250,'utf-8')
                                ]);break;
                        }
                        if ($updateResult) {
                            $this->apiService->removePlatformDataEntries('invoice_callBack_' . $invoiceResponse['ddlsh'] . '_applyInvoice');
                        }
                    }
                }
            }catch(\Throwable $exception){
                SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':'.__FUNCTION__, 'error');
            }
        }
    }

    public function testEmail($params) {
        $updatedInvoiceResult = $this->invoiceRecordRepository->getInvoiceRecordByNumber($params);
        $this->apiService->pushQueue([
            'params' => current($updatedInvoiceResult),
            'consumer' => __CLASS__,
            'method' => 'sendInvoiceEmail'
        ],'invoiceEmailPush');
    }

    public function sendInvoiceEmail($invoiceParams)
    {
        $sended = $this->apiService->getPlatformDataEntries('invoice_sended_mail_list','data_values');
        if($sended){
            if(in_array($invoiceParams['invoice_real_number'],$sended)){
                return true;
            }
        }
        $sended = !$sended ? [] :$sended;

        //获取发票地址
        if(isset($invoiceParams['invoice_pdf_url']) && !empty($invoiceParams['invoice_pdf_url'])){
            $filename = $invoiceParams['platform_invoice_id'].'_'.$invoiceParams['invoice_number'].'_'.$invoiceParams['invoice_real_date'].'.pdf';
            $path = config('filesystems.default.root').'public/finance/invoice/'.$invoiceParams['invoice_platform'].'/'.$invoiceParams['platform_invoice_id'].'/';
            $this->saveInvoicePdf($path.$filename, $invoiceParams['invoice_pdf_url']);
        }else{
            $this->getDownloadUrl($invoiceParams);
            $invoiceResult = $this->invoiceRecordRepository->getInvoiceRecordByNumber($invoiceParams['invoice_number']);
            $invoiceResult = current($invoiceResult);
            if(empty($invoiceResult['invoice_pdf_url'])){
                $this->apiService->pushQueue([
                    'params' => $invoiceResult,
                    'consumer' => __CLASS__,
                    'method' => 'sendInvoiceEmail'
                ],'invoiceEmailPush',20*60);
                return true;
            }
        }
        //生成download发票的url
        $urlParams = [
            'platform_invoice_id' => $invoiceParams['platform_invoice_id'],
            'invoice_real_number' => $invoiceParams['invoice_real_number'],
            'invoice_real_date' => $invoiceParams['invoice_real_date']
        ];
        $token = hash_hmac("sha256", json_encode($urlParams), $invoiceParams['invoice_real_number'], false);
        $downloadUrl = 'http://openapi.chem-site.com/finance/invoice/download?token='.$this->apiService->base64UrlEncode('hash='.$token.'&hm='.$invoiceParams['invoice_real_number']);

        $customerInfo = json_decode($invoiceParams['invoice_customer'],true);

        $title = $customerInfo['cust_name'].'电子发票';

        $platformName = $invoiceParams['invoice_platform'] == 'rhawn' ? '上海易恩化学技术有限公司' : '上海百舜生物科技有限公司';

        $message = '<html><head><meta http-equiv="Content-Type" content="text/html;charset=utf-8"></head><bodyclass="">';

        $message .= "<div>";
        $message .= '尊敬的<span style="font-color: #C33203;font-weight: bold">'.$customerInfo['cust_name'].'</span>,<span style="font-color: #C33203;font-weight: bold">'. $platformName . '</span>给您开具了电子发票，请查收。';
        $message .= "<br /><h3 style='border-bottom: 1px solid #dbdbdb;'>发票信息:</h3><br />";
        $message .= "<span style='height:55px;background-color: #EDF3F5'>开票日期 : ".date('Y-m-d H:i:s',$invoiceParams['invoice_real_date'])."</span><br/>";
        $message .= "<span style='height:55px;background-color: #EDF3F5'>销方名称 : ".$platformName."</span><br />";
        $message .= "<span style='height:55px;background-color: #EDF3F5'>购方名称 : ".$customerInfo['cust_name']."</span><br/>";
        $message .= "<span style='height:55px;background-color: #EDF3F5'>合计金额 : ".bcmul($invoiceParams['invoice_money'],1,2)."</span><br />";
        $message .= "<span style='height:55px;background-color: #EDF3F5'>发票号码 : ".$invoiceParams['invoice_real_number']."</span><br /><br />";

        $message .= "<a href='".$downloadUrl."' style='border-bottom: 1px solid #EDF3F5;'>电子发票下载地址</a>";
        $message .= "</div></body></html>";

        //$html = file_get_contents(resource_path('views/emails/invoice.blade.php'));

        //查询购买方邮箱
        switch($invoiceParams['invoice_platform']){
            case 'rhawn' :
                $cusInfo = $this->rhawnInvoiceRepository->queryCustomerInfoByInvoiceId($invoiceParams['platform_invoice_id']);break;
            case 'bhua' :
                $cusInfo = $this->bhuaInvoiceRepository->queryCustomerInfoByInvoiceId($invoiceParams['platform_invoice_id']);break;
        }
        if($cusInfo){
            $cusInfo = current($cusInfo);
            $this->mailService->sendMailToHtml($title, $cusInfo['cus_invoice_email'],$message);

            $this->apiService->storePlatformDataEntries('invoice_sended_mail_list',array_merge($sended,[$invoiceParams['invoice_real_number']]));
        }
    }

    public function downLoadInvoice($params)
    {
        if(empty($params)){
            return false;
        }
        $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByRealNumber($params['hm']);
        if(!$invoiceInfo){
            throw new \Exception('没有对应的电子发票提供下载',502);
        }
        $invoiceInfo = current($invoiceInfo);
        //验证token
        $urlParams = [
            'platform_invoice_id' => $invoiceInfo['platform_invoice_id'],
            'invoice_real_number' => $invoiceInfo['invoice_real_number'],
            'invoice_real_date' => $invoiceInfo['invoice_real_date']
        ];
        $token = hash_hmac("sha256", json_encode($urlParams), $invoiceInfo['invoice_real_number'], false);
        /*if($token != $params['token']){
            throw new \Exception('电子发票无效无法提供下载',502);
        }*/
        $downloadUrl = $this->getDownloadUrl($invoiceInfo);
        return [ 'path' => $downloadUrl['downloadUrl'], 'filename' => $downloadUrl['filename']];
    }

    private function saveInvoicePdf($path,$pdfUrl = '')
    {
        $cmd = 'chmod -R 777 storage/';
        trim(shell_exec("$cmd 2>&1"));

        if(!Storage::exists($path)){
            if(empty($pdfUrl)){
                SimpleLogs::writeLog('pdf下载地址为空',__CLASS__.':'.__FUNCTION__,'error');
                return false;
            }
            $pdf = $this->apiService->clientRequest('get',$pdfUrl,[]);
            if($pdf){
                Storage::put($path, $pdf);
            }
        }
    }

    private function getDownloadUrl($invoiceInfo)
    {
        if(empty($invoiceInfo['invoice_pdf_url'])){
            $invoiceResponse = $this->queryInvoiceInfo([
                'invoice_platform' => $invoiceInfo['invoice_platform'],
                'invoice_number' => $invoiceInfo['invoice_number']
            ]);
            $updatedInvoiceResult = $this->invoiceRecordRepository->saveInvoice([
                'invoice_number' => $invoiceInfo['invoice_number'],
                'invoice_pdf_url' => $invoiceResponse['pdf'],
            ]);
            $invoiceInfo['invoice_pdf_url'] = $updatedInvoiceResult['invoice_pdf_url'];
        }

        $filename = $invoiceInfo['platform_invoice_id'].'_'.$invoiceInfo['invoice_number'].'_'.$invoiceInfo['invoice_real_date'].'.pdf';
        $path = config('filesystems.default.root').'public/finance/invoice/'.$invoiceInfo['invoice_platform'].'/'.$invoiceInfo['platform_invoice_id'].'/';
        $this->saveInvoicePdf($path.$filename, $invoiceInfo['invoice_pdf_url']);

        $downloadUrl = $invoiceInfo['invoice_platform'] == 'rhawn' ?  'http://download.rhawn.cn/' : 'http://download.chem-mall.com/';

        return [ 'downloadUrl' => $downloadUrl.$path.$filename, 'path' => $path, 'filename' => $filename ];
    }

    public function getInvoicePdfDownloadUrl($params)
    {
        if(empty($params)){
            return false;
        }
        $invoiceInfo = $this->invoiceRecordRepository->getInvoiceRecordByRealNumber($params['invoice_real_number']);
        if(!$invoiceInfo){

            $filename = $params['invoice_real_number'].'.pdf';
            $path = config('filesystems.default.root').'public/finance/invoice/manual/';
            $this->saveInvoicePdf($path.$filename);

            $downloadUrl = 'http://download.rhawn.cn/';

            return [ 'downloadUrl' => $downloadUrl.$path.$filename, 'path' => $path, 'filename' => $filename ];
            //throw new \Exception('没有对应的电子发票提供下载',502);
        }else{
            $invoiceInfo = current($invoiceInfo);
            $downloadUrl = $this->getDownloadUrl($invoiceInfo);
        }

        return $downloadUrl;
    }

    public function resetApplyInvoice($invoiceNumber)
    {
        $invoice = $this->invoiceRecordRepository->getInvoiceRecordByNumber($invoiceNumber);
        if(!$invoice){
            throw new \Exception('该发票流水号不存在',502);
        }
        $invoice = current($invoice);
        try{
            //设置发票平台
            $this->apiService->setInvoicePlatform($invoice['invoice_platform']);

            $values = $this->apiService->getPlatformDataEntries('invoice_apply_'.$invoiceNumber,'data_values');
            /*if($values){
                foreach($values['detailParam'] as &$detail){
                    $detail['spmc'] = $this->charsetToGBK($detail['spmc']);
                }
            }*/
            $this->apiService->applyElecInvoice($values);
            return true;
        }catch(\Throwable $exception){
            $update = [];
            $update['invoice_number'] = $invoice['invoice_number'];
            $update['api_response'] = $exception->getMessage();
            $this->saveInvoiceRecord($update);
            throw $exception;
            //SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':'.__FUNCTION__,'error');
            //var_dump($exception->getMessage());
        }
    }

   public function manualDownLoadInvoicePdf($params)
   {
       $downloadUrl = base64_decode($params);
       if(!$downloadUrl){
           throw new \Exception('不是有效的下载地址');
       }

       $downloadUrl = urldecode($downloadUrl);
       $uriList = explode('/',$downloadUrl);
       $filename = '';
       foreach($uriList as $uri){
           if(strstr($uri,'.pdf')){
               $nameList = explode('_',str_replace(strstr($uri,'.pdf'),'',$uri));
               $filename = $nameList[1].'.pdf';
           }
       }
       $path = config('filesystems.default.root').'public/finance/invoice/manual/';
       $this->saveInvoicePdf($path.$filename, $downloadUrl);

   }

   public function getInvoiceRecordList($requestParams)
   {
       $offset = !isset($requestParams['page_size']) || empty($requestParams['page_size']) ? 200 : $requestParams['page_size'];
       if($offset > 200){
           $offset = 200;
       }
       $pageNo = !isset($requestParams['page_no']) || empty($requestParams['page_no']) ? 1 : $requestParams['page_no'];
       $limit = $pageNo == 1 ? 0 : $pageNo * $offset;
       try{
           $where = [];

           if(isset($requestParams['invoice_number']) && !empty($requestParams['invoice_number'])){
               $where['invoice_number'] = $requestParams['invoice_number'];
           }

           if(isset($requestParams['invoice_real_number']) && !empty($requestParams['invoice_real_number'])){
               $where['invoice_real_number'] = $requestParams['invoice_real_number'];
           }

           if(isset($requestParams['invoice_platform_id']) && !empty($requestParams['invoice_platform_id'])){
               $where['platform_invoice_id'] = $requestParams['invoice_platform_id'];
           }

           if(isset($requestParams['invoice_created_at']) && !empty($requestParams['invoice_created_at'])){
               $where['created_at'] = date('Y-m-d',strtotime($requestParams['invoice_created_at']));
           }

           $invoiceList = $this->invoiceRecordRepository->getList($where,$limit,$offset);
           if($invoiceList){
               $invoiceList = app(InvoiceRecordTransformer::class)->transform($invoiceList);
           }
           $invocieTotal = $this->invoiceRecordRepository->counts($where);

           return $this->invoiceRecordRepository->pagePaginate($invoiceList,$invocieTotal,$requestParams);


           /*$dataReturn = [];
           $dataReturn['data'] = $invoiceList;
           $dataReturn['total'] = $invocieTotal;
           $total = ceil($invocieTotal / $offset) - 1;
           $dataReturn['total_page'] = ceil($invocieTotal / $offset) - 1 == 0 ? 1;
           $dataReturn['page_no'] = $pageNo;*/

       }catch(\Throwable $exception){
           SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':'.__FUNCTION__,'error');
           throw $exception;
       }
   }

   public function getInvoiceStatusDetailByInvoiceNumber($requestParams)
   {
       if(!isset($requestParams['invoice_number']) || empty($requestParams['invoice_number'])){
           throw new \Exception('发票流水号为空',502);
       }

       $invoiceResult = $this->invoiceRecordRepository->getInvoiceRecordByNumber($requestParams);
       if(!$invoiceResult){
           throw new \Exception('该发票流水号不存在',502);
       }
       $invoiceResult = current($invoiceResult);

       //查询该发票接口请求参数
       $values = $this->apiService->getPlatformDataEntries('invoice_apply_'.$requestParams['invoice_number'],'data_values');
       $invoiceResult = array_merge($invoiceResult,['params' => $values]);

       return app(InvoiceRecordStatusDetailTransformer::class)->transform([$invoiceResult]);
   }


   public function delInvoiceRecord($requestParams)
   {
       if(!isset($requestParams['invoice_number']) || empty($requestParams['invoice_number'])){
           throw new \Exception('发票流水号为空',502);
       }

       $invoiceResult = $this->invoiceRecordRepository->getInvoiceRecordByNumber($requestParams);
       if(!$invoiceResult){
           throw new \Exception('该发票流水号不存在',502);
       }
       $invoiceResult = current($invoiceResult);

       $result = $this->invoiceRecordRepository->delInvoiceByNumber($invoiceResult['invoice_number']);
       if($result){
           $this->apiService->removePlatformDataEntries('invoice_apply_'.$invoiceResult['invoice_number']);
       }

       return true;
   }


}
