<?php

namespace App\Http\Controllers\V1\Bhua;

use App\Services\Api\BhuaProductsService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;

class ProductsController extends Controller
{
    use Helpers;

    public function __construct(BhuaProductsService $bhuaProductService)
    {
       $this->bhuaProdcutService = $bhuaProductService;
       $this->controllerType = 'bh';
    }


    public function getAllBrands(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);

        try{
            $brandsList = $this->bhuaProdcutService->getAllBrands($requestParams);
            if($brandsList){
                $brandsList = $this->formatKeysfromArray($brandsList,'toCamelCase');
            }
            return Response::success($brandsList,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取品牌列表失败');
        }
    }

    /**
     *
     * @Post(
     *     path="/openapi/bhua/products/getProductsByBrandId",
     *     tags={"百化 - 产品相关接口"},
     *     summary="获取品牌产品列表",
     *     description="获取品牌产品列表 品牌id 允许范围【
    罗恩：40
    阿拉丁：1
    毕得：10
    乐研 ：108
    九鼎 ：42】",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"brandId","pageSize", "pageNo"},
     *                 @Property(property="brandId", @Schema(type="integer"),description="品牌id 允许范围【
    罗恩：40
    阿拉丁：1
    毕得：10
    乐研 ：108
    九鼎 ：42】"),
     *                 @Property(property="pageSize", @Schema(type="integer"),description="返回数据集数量"),
     *                 @Property(property="pageNo", @Schema(type="integer"),description="当前页数"),
     *                 example={"brandId" : 1,"pageSize" : 100, "pageNo" : 1}
     *             ),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/BhuaProductsList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getProductsByBrand(Request $request)
    {
        $brandId = $request->get('brandId');
        if(!$brandId){
            return Response::ok('品牌id不存在!');
        }

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');

        try{
            $productsList = $this->bhuaProdcutService->getProductsByBrandId($requestParams);
            if($productsList){
                $productsList = $this->formatKeysfromArray($productsList,'toCamelCase');
            }
            return Response::success($productsList,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取品牌产品列表失败');
        }
    }

}
