<?php

namespace App\Rhawn\Repositories\Eloquent;

use App\Rhawn\Repositories\Contracts\RhawnProductRepository;
use App\Rhawn\Repositories\Models\RhawnProducts;
use Illuminate\Support\Facades\DB;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class RhawnProductRepositoryEloquent extends BaseRepository implements RhawnProductRepository
{
    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return RhawnProducts::class;
    }

    /**
     * 根据p_code获取产品信息
     * @param $pCode
     * @return null
     */
    public function getProductByPcode($pCode)
    {
        $productsModel = $this->join('chemicals', 'chemicals.c_id','products.c_id')->join('raw', 'raw.r_id','products.r_id');
        if(!is_array($pCode)){
            $productsModel->where('products.p_code',$pCode);
        }else{
            $productsModel->whereIn('products.p_code',$pCode);
        }
        $product = $productsModel->get();
        if($product){
            return $product->toArray();
        }
        return null;
    }

    public function getProductsByWhere(array $where)
    {
        $productsModel = $this->join('chemicals', 'chemicals.c_id','products.c_id');
        foreach($where as $key => $w){
            if(!is_array($w)){
                $productsModel->where($key,$w);
            }else{
                $productsModel->whereIn($key,$w);
            }
        }
        $product = $productsModel->get();
        if($product){
            return $product->toArray();
        }
        return null;
    }

    /**
     * 根据pid获取产品对应的促销信息
     * @param $pId
     * @return array|null
     */
    public function getProductPromotionByPid($pId)
    {
        $promotionsModel = DB::connection($this->getConnectionName())
            ->table('promotions_products')
            ->join('promotions','promotions.pt_id','promotions_products.pt_id');
        if(is_array($pId)){
            $promotionsModel->whereIn('promotions_products.p_id',$pId);
        }else{
            $promotionsModel->where('promotions_products.p_id',$pId);
        }
        /*$promotions = $promotionsModel->where('promotions.pt_start','<=',time())
            ->where('promotions.pt_end','>=',time())
            ->get();*/
        $promotions = $promotionsModel->get();
        if($promotions){
            $promotions = $promotions->toArray();
            return $promotions;
        }
        return null;
    }

    /**
     * 获取所有产品的包装规格
     * @return array
     */
    public function getProductPackagesThroughGroupByPackUnit()
    {
        $packages = RhawnProducts::query()
            ->select('p_pack','p_pack_unit')
            ->groupBy('p_pack','p_pack_unit')
            ->get();

        return $packages->toArray();
    }

    /**
     * 获取产品信息
     * @param $where
     * @param $offset
     * @param $pageSize
     * @return null
     */
    public function getProductsList($where, $offset, $limit)
    {
        $productsModel = $this->join('chemicals', 'chemicals.c_id','products.c_id')->join('raw', 'raw.r_id','products.r_id');
        if(!empty($where)){
            foreach($where as $key => $w){
                if(!is_array($w)){
                    $productsModel->where($key,$w);
                }else{
                    $productsModel->whereIn($key,$w);
                }
            }
        }
        $product = $productsModel->offset($offset)->limit($limit)->get();
        if($product){
            return $product->toArray();
        }
        return null;
    }

    public function getProductsByCas($cas)
    {
        $productsModel = $this->join('chemicals', 'chemicals.c_id','products.c_id')->join('raw', 'raw.r_id','products.r_id');
        $product = $productsModel->where('chemicals.c_cas',$cas)->get();
        if($product){
            return $product->toArray();
        }
        return null;
    }

    public function getProductIsHotByPid($pid)
    {
        $result =DB::connection($this->getConnectionName())
            ->select(
                DB::raw("SELECT sum(`pstk_num` - `pstk_lock_num`) as `real_num` FROM `pstock` where `si_ids` = '' AND `p_id` = ".$pid)
            );
        if(!empty($result)){
            return $result;
        }
        return null;
    }
}
