<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Rhawn\Services;

use App\Repositories\Eloquent\RhawnSordersRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnCustomerRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnProductRepositoryEloquent;
use App\Rhawn\Repositories\Eloquent\RhawnSoitemsRepositoryEloquent;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class RhawnOrdersService
{
    public function __construct()
    {
        $this->rhawnSorderRepository = app(RhawnSordersRepositoryEloquent::class);
        $this->rhawnSorderItemsRepository = app(RhawnSoitemsRepositoryEloquent::class);
    }

    public function getCustomerOrderItems($cusCode, $soItems)
    {
        if(empty($soItems)){
            throw new \Exception('订单项为空',500);
        }
        $customerReopository = app(RhawnCustomerRepositoryEloquent::class);
        $discount = $customerReopository->getCustomerDiscountByCusCode($cusCode);

        $total = 0;
        $weight = 0;
        $productRepository = app(RhawnProductRepositoryEloquent::class);
        foreach($soItems as $k => $v){
            $productInfo = $productRepository->getProductByPcode($v['p_code']);
            if(!$productInfo){
                throw new \Exception($v['p_code'].'产品不存在!',500);
            }
            $productInfo = current($productInfo);
            if($productInfo['p_status'] == 0 || $productInfo['p_show'] == 0){
                throw new \Exception($v['p_code'].'产品已下架!',500);
            }

            if(isset($discount[$productInfo['b_id']])){
                $productInfo['p_discount'] = round($discount[$productInfo['b_id']]['cd_discount'] * $productInfo['p_price']);
            }else{
                $productInfo['p_discount'] = $productInfo['p_price'];
            }

            if(isset($v['si_molbase_pog'])){
                $soItems[$k]['si_molbase_pog'] = $v['si_molbase_pog'];
            }

            //当前产品是否在促销设定里
            $promotions = $productRepository->getProductPromotionByPid($productInfo['p_id']);
            if($promotions){
                foreach($promotions as $promotion){
                    if($promotion->pt_discount != 0){
                        $productInfo['p_discount'] = round($promotion->pt_discount * $productInfo['p_discount']);
                    }
                }
            }

            $soItems[$k]['amount'] = round(bcmul($productInfo['p_discount'],$v['num'],3),2);
            $soItems[$k]['weight'] = round(bcmul($productInfo['p_weight'],$v['num'],3),3);
            //得到品牌
            $brands = DB::connection($this->rhawnSorderRepository->getConnectionName())
                ->table('brands')
                ->where('b_id',$productInfo['b_id'])
                ->first();
            if(!$brands){
                throw new \Exception('品牌不存在',500);
            }
            $productInfo['b_cn_name'] = $brands->b_cn_name;

            $soItems[$k]['product_info'] = $productInfo;
            $total += $soItems[$k]['amount'];
            $weight += $soItems[$k]['weight'];
        }

        $returnItems = [];
        $returnItems['total'] = $total;
        $returnItems['weight'] = $weight;
        $returnItems['rows'] = $soItems;

        return $returnItems;
    }

    /**
     * 新增订单
     * @param $data
     * @return false
     */
    public function createOrders($data)
    {
        $connection = DB::connection($this->rhawnSorderRepository->getConnectionName());
        try{
           $connection->beginTransaction();

            $data['sorders']['so_no'] = '';
            $data['sorders']['so_ctime'] = time();
            $data['sorders']['so_creater'] = 0;
            $data['sorders']['so_standard'] = '';

            $orderId = $connection->table('sorders')->insertGetId($data['sorders']);

            $soNo = 1000000 + $orderId;
            $soNo = date('Ymd',time()).'-'.$soNo;

            $connection->table('sorders')
                ->where('so_id',$orderId)
                ->update([
                    'so_no' => $soNo
                ]);

            $soitems = [];
            foreach($data['soitems'] as $k => $v){
                $soitems[$k]['so_id'] = $orderId;
                $soitems[$k]['p_id'] = $v['p_id'];
                $soitems[$k]['si_price'] = $v['si_price'];
                $soitems[$k]['si_discount'] = $v['si_discount'];
                $soitems[$k]['si_num'] = $v['si_num'];
                $soitems[$k]['si_amount'] = $v['si_amount'];
                $soitems[$k]['si_vamount'] = $v['si_vamount'];
                $soitems[$k]['si_p_tod'] = $v['si_p_tod'];
                if(isset($v['si_cus_note'])){
                    $soitems[$k]['si_cus_note'] = $v['si_cus_note'];
                }else{
                    $soitems[$k]['si_cus_note'] = '';
                }
                if(isset($v['si_molbase_pog'])){
                    $soitems[$k]['si_molbase_pog'] = $v['si_molbase_pog'];
                }else{
                    $soitems[$k]['si_molbase_pog'] = '';
                }
                $soitems[$k]['si_nodes'] = '';
                $soitems[$k]['si_notes'] = '';
                $soitems[$k]['si_sales_note'] = '';
            }
            if($data['express'] > 0){
                $soitems_count = count($soitems);
                $soitems[$soitems_count]['so_id'] = $orderId;
                $soitems[$soitems_count]['p_id'] = 0;
                $soitems[$soitems_count]['si_price'] = 0;
                $soitems[$soitems_count]['si_discount'] = 0;
                $soitems[$soitems_count]['si_num'] = 0;
                $soitems[$soitems_count]['si_amount'] = $data['express'];
                $soitems[$soitems_count]['si_vamount'] = 0;
                $soitems[$soitems_count]['si_p_tod'] = '';
                $soitems[$soitems_count]['si_cus_note'] = '';
                $soitems[$soitems_count]['si_molbase_pog'] = '';
                $soitems[$soitems_count]['si_nodes'] = '';
                $soitems[$soitems_count]['si_notes'] = '';
                $soitems[$soitems_count]['si_sales_note'] = '';
            }

            $connection->table('soitems')->insert($soitems);
            if($data['v_id'] != ''){
                $orderId = $connection->table('voucher')
                    ->where('v_id',$data['v_id'])
                    ->update([
                        'so_id' => $orderId
                    ]);
            }

            $connection->commit();

            return $orderId;

        }catch(\Throwable $exception){
            $connection->rollBack();
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':createOrders','error');
            throw $exception;
        }
    }

    /**
     * 计算运费
     * @param $weight
     * @param $province
     * @param $total
     * @param $if_free_express
     * @param $cus_level
     * @return float|int
     */
    public function getExpress($weight,$province,$total,$if_free_express,$cus_level){
        //计算运费 (运费=首重（1kg）*运输系数（首重）+续重（总重量-1kg）*运输系数（续重）)
        $express = 0;
        if($if_free_express == 1 || $cus_level == 3){
            return $express;
        }else{
            if($cus_level == 0){
                $max_amount = 150;
            }
            if($cus_level == 1){
                $max_amount = 99;
            }
            if($cus_level == 2){
                $max_amount = 66;
            }
            //统一调整为满66免运费
            //$max_amount = 66;
            if($weight == 0){
                $weight = 0.1;
            }
            if($total < $max_amount && $weight > 0){
                $weight = ceil($weight);
                if($province == '江苏省' || $province == '浙江省' || $province == '上海' || $province == '上海市'){
                    $express = ($weight - 1)*2 + 6;
                }else{
                    $express = ($weight - 1)*8 + 10;
                }
            }
            return $express;
        }
    }

    public function getCustomerOrderItemsByOrderId($cusId, $orderId)
    {
        return $this->rhawnSorderRepository->getCustomerOrderItemsByOrderId($cusId, $orderId);
    }

    public function getSordersByOrderNo($orderNo)
    {
       $order = $this->rhawnSorderRepository->getSorderFromOrderNo($orderNo);
       if($order){
           return $order->toArray();
       }
       return null;
    }

    /**
     * 取消客户订单
     * @param $cusId
     * @param $orderId
     * @return bool
     * @throws \Throwable
     */
    public function cancelCustomerOrderByOrderId($cusId, $orderId)
    {
        $connection = $this->rhawnSorderRepository->getConnection();
        try{
            $connection->beginTransaction();

            $connection->table('sorders')
                ->where('so_id',$orderId)
                ->where('cus_id',$cusId)
                ->update([
                    'so_review_status'=>'3','so_total'=>'0'
                ]);

            $connection->table('soitems')
                ->where('so_id',$orderId)
                ->update([
                    'si_if_cancel'=>'1'
                ]);

            $connection->commit();

            return true;
        }catch(\Throwable $exception){
            $connection->rollBack();
            throw $exception;
        }
    }

    /**
     * 获取订单配货单详情
     * @param $orderNumber
     */
    public function getOrderDispatchDetail($orderNumber)
    {
        try{
            $dpdetail = $this->rhawnSorderRepository->getOrderDispatch($orderNumber);
            $soItemIds = [];
            if($dpdetail){
                foreach($dpdetail as $detail){
                    $soItemIds[] = $detail['si_id'];
                }
                $soItems = app(RhawnSoitemsRepositoryEloquent::class)->getSorderItemFromItemId($soItemIds);
                if(!$soItems){
                    throw new \Exception('订单项不存在',500);
                }
                $orderItems = [];
                foreach($soItems as $item){
                    $orderItems[$item['si_id']] = $item;
                }
                foreach($dpdetail as &$detail){
                    if(isset($orderItems[$detail['si_id']]) && !empty($orderItems[$detail['si_id']])){
                        $detail['si_id'] = $orderItems[$detail['si_id']];
                    }
                }
            }
            return $dpdetail;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getOrderDispatchDetail','error');
            throw $exception;
        }
    }
}
