<?php

namespace App\Services\Api;

use App\Repositories\Transformers\Rhawn\Bhua\BrandsTransformer;
use App\Repositories\Transformers\Rhawn\Bhua\ProductsTransformer;
use App\Rhawn\Services\BhuaCustomerService;
use App\Rhawn\Services\BhuaProductService;
use App\Support\Facades\SimpleLogs;
use Illuminate\Support\Facades\DB;

class BhuaProductsService
{
    public function __construct()
    {
        $this->bhuaProductService = app(BhuaProductService::class);
        $this->bhuaCustomerService = app(BhuaCustomerService::class);
    }

    public function getAllBrands($requestParams)
    {
        $offset = !isset($requestParams['page_size']) || empty($requestParams['page_size']) ? 2000 : $requestParams['page_size'];
        if($offset > 200){
            $offset = 200;
        }
        $pageNo = !isset($requestParams['page_no']) || empty($requestParams['page_no']) ? 1 : $requestParams['page_no'];
        $limit = $pageNo == 1 ? 0 : $pageNo * $offset;
        try{
            $where = [];
            $brandsList = $this->bhuaProductService->getBrandsList($where,$offset,$limit);
            $brandsTotal = 0;
            if($brandsList){
                $brandsList = app(BrandsTransformer::class)->transform($brandsList);
                $brandsTotal = $this->bhuaProductService->brandsRepository->count($where);
            }

            $dataReturn = [];
            $dataReturn['data'] = $brandsList;
            $dataReturn['total_page'] = ceil($brandsTotal / $offset);
            $dataReturn['page_no'] = $pageNo;

            return $dataReturn;
        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getAllBrands','error');
            throw $exception;
        }
    }

    public function getProductsByBrandId($params)
    {
        if(!isset($params['brand_id']) || empty($params['brand_id'])){
            throw new \Exception('品牌id必传',502);
        }
        if(!is_numeric($params['brand_id'])){
            throw new \Exception('品牌id类型错误',502);
        }
        $params['brand_id'] = intval($params['brand_id']);

        if(!in_array($params['brand_id'],[40,1,10,108,42])){
            throw new \Exception('品牌id不在授权范围之内',502);
        }

        $offset = !isset($params['page_size']) || empty($params['page_size']) ? 200 : $params['page_size'];
        if($offset > 200){
            $offset = 200;
        }
        $pageNo = !isset($params['page_no']) || empty($params['page_no']) ? 1 : $params['page_no'];
        $limit = $pageNo == 1 ? 0 : $pageNo * $offset;

        try{
            $brand = $this->bhuaProductService->brandsRepository->getBrandById($params['brand_id']);
            if(!$brand){
                throw new \Exception('没找到相应品牌id',502);
            }
            $where = [
                'b_id' => $params['brand_id'],
                'p_status' => 1,
                'p_show' => 1
            ];
            $products = $this->bhuaProductService->productRepository->getProductsListByBrandId($params['brand_id'], $limit, $offset);
            $total = 0;
            if($products){
                $products = app(ProductsTransformer::class)->transform($products);
            }
            $total = $this->bhuaProductService->productRepository->count($where);

            $dataReturn = [];
            $dataReturn['data'] = $products;
            $dataReturn['total_page'] = ceil($total / $offset) - 1;
            $dataReturn['page_no'] = $pageNo;

            return $dataReturn;

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),__CLASS__.':getProductsByBrandId','error');
            throw $exception;
        }
    }



}
